library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use font.all;

entity calendar200708 is
	port(
		CLK_50MHZ : in std_logic;
		SW : in std_logic_vector (3 downto 0);
		BTN_NORTH : in std_logic;
		BTN_SOUTH : in std_logic;
		BTN_WEST : in std_logic;
		BTN_EAST : in std_logic;
		ROT_CENTER : in std_logic;
		VGA_RED : out std_logic;
		VGA_GREEN : out std_logic;
		VGA_BLUE : out std_logic;
		VGA_HSYNC : out std_logic;
		VGA_VSYNC : out std_logic;
		LED : out std_logic_vector (7 downto 0));
end calendar200708;

architecture behavioral of calendar200708 is
	constant H_ACTIVE_PIXEL_LIMIT : Integer := 640;
	constant H_FPORCH_PIXEL_LIMIT : Integer := 656; -- 640+16
	constant H_SYNC_PIXEL_LIMIT   : Integer := 752; -- 640+16+96
	constant H_BPORCH_PIXEL_LIMIT : Integer := 800; -- 640+16+96+48

	constant V_ACTIVE_LINE_LIMIT : Integer := 480;
	constant V_FPORCH_LINE_LIMIT : Integer := 490; -- 480+10
	constant V_SYNC_LINE_LIMIT   : Integer := 492; -- 480+10+2
	constant V_BPORCH_LINE_LIMIT : Integer := 521; -- 480+10+2+29

	constant url_x : Integer := 40;
	constant url_x_pitch : Integer := 16;

	constant calendar_x : integer := 96;
	constant calendar_x_pitch : integer := 64;

	signal CLK_25MHZ : std_logic;
	signal vga_r : std_logic;
	signal vga_g : std_logic;
	signal vga_b : std_logic;
	signal vga_hsync_reg : std_logic;
	signal vga_vsync_reg : std_logic;
	signal hcount : integer;
	signal vcount : integer;
	signal url_pattern : font_table;
	signal date_pattern : font_table;
begin
	LED <= "00000000";

	VGA_RED <= vga_r;
	VGA_GREEN <= vga_g;
	VGA_BLUE <= vga_b;
	VGA_HSYNC <= vga_hsync_reg;
	VGA_VSYNC <= vga_vsync_reg;

	process 
	begin
		wait until CLK_50MHZ'event and CLK_50MHZ = '1';
		CLK_25MHZ <= not CLK_25MHZ;
	end process;

	process
		procedure text_out(
			display : out std_logic;
			x : in integer;
			y : in integer;
			pattern : in font_table) is
		begin
			if y >= 0 and y < 24
				and x >= 0 and x < 16
			then
				display := pattern (y * 16 + x);
			else
				display := '0';
			end if;
		end text_out;
		variable d : std_logic;
		variable date : integer;
		variable y : integer;
		variable r,g,b : std_logic;
	begin
		wait until CLK_25MHZ'event and CLK_25MHZ = '1';

		if hcount < H_BPORCH_PIXEL_LIMIT then
			hcount <= hcount + 1;
		else
			hcount <= 0;
			if vcount < V_BPORCH_LINE_LIMIT then
				vcount <= vcount + 1;
			else 
				vcount <= 0;
			end if;
		end if;
	
		if hcount < H_ACTIVE_PIXEL_LIMIT
			and vcount < V_ACTIVE_LINE_LIMIT
		then
			vga_b <= '0';
			vga_r <= '0';
			if (hcount + vcount) mod 2 = 0 then
				vga_g <= '0';
			else
				vga_g <= '1';
			end if;

			text_out(d, hcount - 240, vcount - 48, font_32);
			if d = '1' then vga_r <= '1'; vga_g <= '1'; vga_b <= '1'; end if;
			text_out(d, hcount - 260, vcount - 48, font_30);
			if d = '1' then vga_r <= '1'; vga_g <= '1'; vga_b <= '1'; end if;
			text_out(d, hcount - 280, vcount - 48, font_30);
			if d = '1' then vga_r <= '1'; vga_g <= '1'; vga_b <= '1'; end if;
			text_out(d, hcount - 300, vcount - 48, font_37);
			if d = '1' then vga_r <= '1'; vga_g <= '1'; vga_b <= '1'; end if;
	
			text_out(d, (hcount - 360) / 2, (vcount - 48) / 2, font_38);
			if d = '1' then vga_r <= '1'; vga_g <= '1'; vga_b <= '1'; end if;
			
			if hcount >= calendar_x and hcount < calendar_x_pitch * 8
				and (hcount - calendar_x) mod calendar_x_pitch < 32 
			then
				if vcount < 96 + 48 * 2 then
					date := (hcount - calendar_x) / calendar_x_pitch - 2;
				elsif vcount < 96 + 48 * 3 then
					date := (hcount - calendar_x) / calendar_x_pitch + 5;
				elsif vcount < 96 + 48 * 4 then
					date := (hcount - calendar_x) / calendar_x_pitch + 12;
				elsif vcount < 96 + 48 * 5 then
					date := (hcount - calendar_x) / calendar_x_pitch + 19;
				else
					date := (hcount - calendar_x) / calendar_x_pitch + 26;
				end if;
				
				if date <= 0 or date >= 32 then
					null;
				else
					case date is
						when 4 | 11 | 18 | 25 =>
							r := '0'; g := '0'; b := '1';
						when 5 | 12 | 19 | 26 =>
							r := '1'; g := '0'; b := '0';
						when others =>
							r := '1'; g := '1'; b := '1';
					end case;

					if date <= 4 then
						y := 96 + 48 * 1;
					elsif date <= 11 then
						y := 96 + 48 * 2;
					elsif date <= 18 then
						y := 96 + 48 * 3;
					elsif date <= 25 then
						y := 96 + 48 * 4;
					else
						y := 96 + 48 * 5;
					end if;
				
					if (hcount - calendar_x) mod calendar_x_pitch < 16 then
						if date < 10 then
							date := 0;
						elsif date < 20 then
							date := 1;
						elsif date < 30 then
							date := 2;
						else
							date := 3;
						end if;
					else
						case date is
							when 1 | 11 | 21 | 31 => date := 1;
							when 2 | 12 | 22 => date := 2;
							when 3 | 13 | 23 => date := 3;
							when 4 | 14 | 24 => date := 4;
							when 5 | 15 | 25 => date := 5;
							when 6 | 16 | 26 => date := 6;
							when 7 | 17 | 27 => date := 7;
							when 8 | 18 | 28 => date := 8;
							when 9 | 19 | 29 => date := 9;
							when others => date := 0;
						end case;
					end if;
				
					case date is
						when 0 => date_pattern <= font_30;
						when 1 => date_pattern <= font_31;
						when 2 => date_pattern <= font_32;
						when 3 => date_pattern <= font_33;
						when 4 => date_pattern <= font_34;
						when 5 => date_pattern <= font_35;
						when 6 => date_pattern <= font_36;
						when 7 => date_pattern <= font_37;
						when 8 => date_pattern <= font_38;
						when 9 => date_pattern <= font_39;
						when others => null;
					end case;
				
					text_out(d, 
						(hcount - calendar_x) mod 16, 
						vcount - y,
						date_pattern);
	
					if d = '1' then vga_r <= r; vga_g <= g; vga_b <= b; end if;
				end if;
			end if;

			if hcount >= url_x and hcount < url_x + url_x_pitch * 36
				and vcount >= 440 and vcount < 440 + 24 
			then
				-- http://panathenaia.halfmoon.jp/vhdl/
				case hcount is
					when url_x + url_x_pitch * 0 => url_pattern <= font_68; -- h
					when url_x + url_x_pitch * 1 => url_pattern <= font_74; -- t
					when url_x + url_x_pitch * 2 => url_pattern <= font_74; -- t
					when url_x + url_x_pitch * 3 => url_pattern <= font_70; -- p
					when url_x + url_x_pitch * 4 => url_pattern <= font_3a; -- :
					when url_x + url_x_pitch * 5 => url_pattern <= font_2f; -- /
					when url_x + url_x_pitch * 6 => url_pattern <= font_2f; -- /
					when url_x + url_x_pitch * 7 => url_pattern <= font_70; -- p
					when url_x + url_x_pitch * 8 => url_pattern <= font_61; -- a
					when url_x + url_x_pitch * 9 => url_pattern <= font_6e; -- n
					when url_x + url_x_pitch * 10 => url_pattern <= font_61; -- a
					when url_x + url_x_pitch * 11 => url_pattern <= font_74; -- t
					when url_x + url_x_pitch * 12 => url_pattern <= font_68; -- h
					when url_x + url_x_pitch * 13 => url_pattern <= font_65; -- e
					when url_x + url_x_pitch * 14 => url_pattern <= font_6e; -- n
					when url_x + url_x_pitch * 15 => url_pattern <= font_61; -- a
					when url_x + url_x_pitch * 16 => url_pattern <= font_69; -- i
					when url_x + url_x_pitch * 17 => url_pattern <= font_61; -- a
					when url_x + url_x_pitch * 18 => url_pattern <= font_2e; -- .
					when url_x + url_x_pitch * 19 => url_pattern <= font_68; -- h
					when url_x + url_x_pitch * 20 => url_pattern <= font_61; -- a
					when url_x + url_x_pitch * 21 => url_pattern <= font_6c; -- l
					when url_x + url_x_pitch * 22 => url_pattern <= font_66; -- f
					when url_x + url_x_pitch * 23 => url_pattern <= font_6d; -- m
					when url_x + url_x_pitch * 24 => url_pattern <= font_6f; -- o
					when url_x + url_x_pitch * 25 => url_pattern <= font_6f; -- o
					when url_x + url_x_pitch * 26 => url_pattern <= font_6e; -- n
					when url_x + url_x_pitch * 27 => url_pattern <= font_2e; -- .
					when url_x + url_x_pitch * 28 => url_pattern <= font_6a; -- j
					when url_x + url_x_pitch * 29 => url_pattern <= font_70; -- p
					when url_x + url_x_pitch * 30 => url_pattern <= font_2f; -- /
					when url_x + url_x_pitch * 31 => url_pattern <= font_76; -- v
					when url_x + url_x_pitch * 32 => url_pattern <= font_68; -- h
					when url_x + url_x_pitch * 33 => url_pattern <= font_64; -- d
					when url_x + url_x_pitch * 34 => url_pattern <= font_6c; -- l
					when url_x + url_x_pitch * 35 => url_pattern <= font_2f; -- /
					when others => null;
				end case;
				text_out(d, (hcount - url_x) mod 16, vcount - 440, url_pattern);
				if d = '1' then vga_r <= '1'; vga_g <= '1'; vga_b <= '1'; end if;
			end if;
		else
			vga_b <= '0';
			vga_r <= '0';
			vga_g <= '0';
		end if;

		case hcount is
			when H_ACTIVE_PIXEL_LIMIT => -- front porch
				null;
			when H_FPORCH_PIXEL_LIMIT => -- sync pulse
				vga_hsync_reg <= '0';
			when H_SYNC_PIXEL_LIMIT => -- back porch
				vga_hsync_reg <= '1';
			when others =>
				null;
		end case;
	
		case vcount is
			when V_ACTIVE_LINE_LIMIT => -- front porch
				null;
			when V_FPORCH_LINE_LIMIT => -- sync pulse
				vga_vsync_reg <= '0';
			when V_SYNC_LINE_LIMIT => -- back porch
				vga_vsync_reg <= '1';
			when others =>
				null;
		end case;
	end process;

end behavioral;
